@@ -1,3 +1,19 @@
+/*
+   Copyright The containerd Authors.
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+*/
+
 package testsuite
 
 import (
@@ -11,12 +27,13 @@ import (
 	"time"
 
 	"github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/fs/fstest"
 	"github.com/containerd/containerd/mount"
 	"github.com/containerd/containerd/namespaces"
+	"github.com/containerd/containerd/pkg/testutil"
 	"github.com/containerd/containerd/snapshots"
-	"github.com/containerd/containerd/testutil"
-	"github.com/stretchr/testify/assert"
+	"github.com/containerd/continuity/fs/fstest"
+	"gotest.tools/assert"
+	is "gotest.tools/assert/cmp"
 )
 
 // SnapshotterSuite runs a test suite on the snapshotter given a factory function.
@@ -145,8 +162,8 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 		t.Fatalf("failure reason: %+v", err)
 	}
 
-	assert.Equal(t, "", si.Parent)
-	assert.Equal(t, snapshots.KindCommitted, si.Kind)
+	assert.Check(t, is.Equal("", si.Parent))
+	assert.Check(t, is.Equal(snapshots.KindCommitted, si.Kind))
 
 	_, err = snapshotter.Stat(ctx, preparing)
 	if err == nil {
@@ -180,8 +197,8 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 		t.Fatal(err)
 	}
 
-	assert.Equal(t, committed, ni.Parent)
-	assert.Equal(t, snapshots.KindActive, ni.Kind)
+	assert.Check(t, is.Equal(committed, ni.Parent))
+	assert.Check(t, is.Equal(snapshots.KindActive, ni.Kind))
 
 	nextCommitted := filepath.Join(work, "committed-next")
 	if err := snapshotter.Commit(ctx, nextCommitted, next, opt); err != nil {
@@ -193,8 +210,8 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 		t.Fatalf("failure reason: %+v", err)
 	}
 
-	assert.Equal(t, committed, si2.Parent)
-	assert.Equal(t, snapshots.KindCommitted, si2.Kind)
+	assert.Check(t, is.Equal(committed, si2.Parent))
+	assert.Check(t, is.Equal(snapshots.KindCommitted, si2.Kind))
 
 	_, err = snapshotter.Stat(ctx, next)
 	if err == nil {
@@ -206,7 +223,7 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 		si2.Name: si2,
 	}
 	walked := map[string]snapshots.Info{} // walk is not ordered
-	assert.NoError(t, snapshotter.Walk(ctx, func(ctx context.Context, si snapshots.Info) error {
+	assert.NilError(t, snapshotter.Walk(ctx, func(ctx context.Context, si snapshots.Info) error {
 		walked[si.Name] = si
 		return nil
 	}))
@@ -217,7 +234,7 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 			t.Errorf("Missing stat for %v", ek)
 			continue
 		}
-		assert.Equal(t, ev, av)
+		assert.Check(t, is.DeepEqual(ev, av))
 	}
 
 	nextnext := filepath.Join(work, "nextnextlayer")
@@ -240,10 +257,10 @@ func checkSnapshotterBasic(ctx context.Context, t *testing.T, snapshotter snapsh
 	}
 
 	testutil.Unmount(t, nextnext)
-	assert.NoError(t, snapshotter.Remove(ctx, nextnext))
-	assert.Error(t, snapshotter.Remove(ctx, committed))
-	assert.NoError(t, snapshotter.Remove(ctx, nextCommitted))
-	assert.NoError(t, snapshotter.Remove(ctx, committed))
+	assert.NilError(t, snapshotter.Remove(ctx, nextnext))
+	assert.Assert(t, is.ErrorContains(snapshotter.Remove(ctx, committed), "remove"))
+	assert.NilError(t, snapshotter.Remove(ctx, nextCommitted))
+	assert.NilError(t, snapshotter.Remove(ctx, committed))
 }
 
 // Create a New Layer on top of base layer with Prepare, Stat on new layer, should return Active layer.
@@ -275,9 +292,9 @@ func checkSnapshotterStatActive(ctx context.Context, t *testing.T, snapshotter s
 	if err != nil {
 		t.Fatal(err)
 	}
-	assert.Equal(t, si.Name, preparing)
-	assert.Equal(t, snapshots.KindActive, si.Kind)
-	assert.Equal(t, "", si.Parent)
+	assert.Check(t, is.Equal(si.Name, preparing))
+	assert.Check(t, is.Equal(snapshots.KindActive, si.Kind))
+	assert.Check(t, is.Equal("", si.Parent))
 }
 
 // Commit a New Layer on top of base layer with Prepare & Commit , Stat on new layer, should return Committed layer.
@@ -314,9 +331,9 @@ func checkSnapshotterStatCommitted(ctx context.Context, t *testing.T, snapshotte
 	if err != nil {
 		t.Fatal(err)
 	}
-	assert.Equal(t, si.Name, committed)
-	assert.Equal(t, snapshots.KindCommitted, si.Kind)
-	assert.Equal(t, "", si.Parent)
+	assert.Check(t, is.Equal(si.Name, committed))
+	assert.Check(t, is.Equal(snapshots.KindCommitted, si.Kind))
+	assert.Check(t, is.Equal("", si.Parent))
 
 }
 
@@ -389,9 +406,9 @@ func checkSnapshotterTransitivity(ctx context.Context, t *testing.T, snapshotter
 	}
 
 	// Test the transivity
-	assert.Equal(t, "", siA.Parent)
-	assert.Equal(t, snapA, siB.Parent)
-	assert.Equal(t, "", siParentB.Parent)
+	assert.Check(t, is.Equal("", siA.Parent))
+	assert.Check(t, is.Equal(snapA, siB.Parent))
+	assert.Check(t, is.Equal("", siParentB.Parent))
 
 }
 
@@ -421,8 +438,7 @@ func checkSnapshotterPrepareView(ctx context.Context, t *testing.T, snapshotter
 	}
 
 	_, err = snapshotter.View(ctx, newLayer, snapA, opt)
-	//must be err != nil
-	assert.NotNil(t, err)
+	assert.Check(t, err != nil)
 
 	// Two Prepare with same key
 	prepLayer := filepath.Join(work, "prepLayer")
@@ -436,8 +452,7 @@ func checkSnapshotterPrepareView(ctx context.Context, t *testing.T, snapshotter
 	}
 
 	_, err = snapshotter.Prepare(ctx, prepLayer, snapA, opt)
-	//must be err != nil
-	assert.NotNil(t, err)
+	assert.Check(t, err != nil)
 
 	// Two View with same key
 	viewLayer := filepath.Join(work, "viewLayer")
@@ -451,8 +466,7 @@ func checkSnapshotterPrepareView(ctx context.Context, t *testing.T, snapshotter
 	}
 
 	_, err = snapshotter.View(ctx, viewLayer, snapA, opt)
-	//must be err != nil
-	assert.NotNil(t, err)
+	assert.Check(t, err != nil)
 
 }
 
@@ -529,7 +543,7 @@ func checkRemoveIntermediateSnapshot(ctx context.Context, t *testing.T, snapshot
 // baseTestSnapshots creates a base set of snapshots for tests, each snapshot is empty
 // Tests snapshots:
 //  c1 - committed snapshot, no parent
-//  c2 - commited snapshot, c1 is parent
+//  c2 - committed snapshot, c1 is parent
 //  a1 - active snapshot, c2 is parent
 //  a1 - active snapshot, no parent
 //  v1 - view snapshot, v1 is parent
@@ -743,7 +757,7 @@ func checkRemove(ctx context.Context, t *testing.T, snapshotter snapshots.Snapsh
 	if err := snapshotter.Remove(ctx, "committed-1"); err != nil {
 		t.Fatal(err)
 	}
-	if err := snapshotter.Commit(ctx, "commited-1", "reuse-1", opt); err != nil {
+	if err := snapshotter.Commit(ctx, "committed-1", "reuse-1", opt); err != nil {
 		t.Fatal(err)
 	}
 }
@@ -755,7 +769,7 @@ func checkSnapshotterViewReadonly(ctx context.Context, t *testing.T, snapshotter
 	if _, err := snapshotter.Prepare(ctx, preparing, "", opt); err != nil {
 		t.Fatal(err)
 	}
-	committed := filepath.Join(work, "commited")
+	committed := filepath.Join(work, "committed")
 	if err := snapshotter.Commit(ctx, committed, preparing, opt); err != nil {
 		t.Fatal(err)
 	}
@@ -781,8 +795,8 @@ func checkSnapshotterViewReadonly(ctx context.Context, t *testing.T, snapshotter
 		t.Fatalf("write to %q should fail (EROFS) but did not fail", testfile)
 	}
 	testutil.Unmount(t, viewMountPoint)
-	assert.NoError(t, snapshotter.Remove(ctx, view))
-	assert.NoError(t, snapshotter.Remove(ctx, committed))
+	assert.NilError(t, snapshotter.Remove(ctx, view))
+	assert.NilError(t, snapshotter.Remove(ctx, committed))
 }
 
 // Move files from base layer to new location in intermediate layer.
